/*
 genx realtime terrain visualization
 (c) 2004-2005 Witold Bolt <ja@hope.art.pl>

 file: heightmap.c

 height map handling

*/

#include <stdio.h>  
#include <stdlib.h>
#include <math.h>
#include <GL/gl.h>

#include "SDL.h"
#include "SDL_image.h"

#include "heightmap.h"
#include "genx.h"
#include "error.h"
#include "mesh.h"

int tSIZE=0;
float SCALE_XZ=0.0f,SCALE_Y=0.0f;

static SDL_Surface *mapa;
static SDL_Surface *heightScale;
static unsigned char *heightMap;      



void freeHeightMap()
{
 SDL_FreeSurface(mapa);
}

int getDataFromFile(char *fName)
{
 char filename[255];

 sprintf(filename,"%s/%s",MAPS_DIR,fName);

#ifdef DEBUG
 fprintf(debug,"debug: starting reading file %s\n",filename);
#endif

 mapa = IMG_Load(filename);
 if (!mapa) {
 fprintf(debug,"ERROR: loading map file %s failed with error: %s\n",filename,IMG_GetError());
 return(-1);
 }
#ifdef DEBUG
 else
 fprintf(debug,"debug: file %s ok:)\n",filename);
#endif

  if(mapa->format->BitsPerPixel!=8)
  { 
    fprintf(debug,"ERROR: height map must be one 8bit channel file!\n");
    return(-1);
  }

  if (mapa->w == mapa->h) {
   tSIZE=mapa->w;
#ifdef DEBUG
   fprintf(debug,"debug: tSIZE = %d\n",tSIZE);
#endif
  }
 
  else   {
   fprintf(debug,"error reding file: %s - your terrain must be square sized!\n",filename);
   return(-1);
  }

   SCALE_XZ = TERRAIN_SIZE_XZ / tSIZE;
   SCALE_Y = TERRAIN_SIZE_Y / 255.0f;

#ifdef DEBUG
   fprintf(debug,"debug: filename: %s\n",filename);
   fprintf(debug,"debug: TERRAIN_SIZE_XZ = %f\n",TERRAIN_SIZE_XZ);
   fprintf(debug,"debug: TERRAIN_SIZE_Y = %f\n",TERRAIN_SIZE_Y);
   fprintf(debug,"debug: SCALE_XZ = %f\n",SCALE_XZ);
   fprintf(debug,"debug: SCALE_Y = %f\n",SCALE_Y);
#endif

 heightMap = mapa->pixels;


#ifdef DEBUG
  fprintf(debug,"debug: done reading file %s\n",filename);
#endif

 return(0);
}

/* returns height at a specific point */
unsigned char getHeight(unsigned short int x, unsigned short int z)
{
 if( (x<tSIZE) && (z<tSIZE) ) return heightMap[x+tSIZE*z];

#ifdef DEBUG
 else {
/* this should not happen */
 fprintf(debug,"ERROR: getHeight(%d,%d) - x or z to big!\n",x,z);
 exit(-1);
 }
#endif

 return(0);
}

int initHeightScale(char *filename)
{
  char _filename[256];

  sprintf(_filename,"%s/%s",TEXTURES_DIR,filename);

  heightScale=IMG_Load(_filename);

  if(!heightScale) {
  fprintf(debug,"ERROR: loading height scale file: %s failed!\n",_filename);
  return(-1);
  }

  if(heightScale->format->BytesPerPixel!=3 && heightScale->format->BytesPerPixel!=4) {
  fprintf(debug,"ERROR: height scale file: %s has wrong format!\n",_filename);
  return(-1);
  }

  return(0);
}

void getHeightPointColor(unsigned short int x, unsigned short int z, float color[4])
{
  unsigned int temp;
  Uint8 *scale;

  temp = getHeight(x,z);

  scale = heightScale->pixels;

  if (heightScale->format->BitsPerPixel == 24) {
  color[0] = (float)scale[3*(temp)] / 255.0f;
  color[1] = (float)scale[3*(temp)+1] / 255.0f;
  color[2] = (float)scale[3*(temp)+2] / 255.0f;
  color[3] = 1.0f;
  } else if (heightScale->format->BitsPerPixel == 32) {
  color[0] = (float)scale[4*(temp)] / 255.0f;
  color[1] = (float)scale[4*(temp)+1] / 255.0f;
  color[2] = (float)scale[4*(temp)+2] / 255.0f;
  color[3] = (float)scale[4*(temp)+3] / 255.0f;
 }

}

void getHeightPointColor8(unsigned short int x, unsigned short int z, Uint8 color[3])
{
  unsigned int temp;
  Uint8 *scale;

  temp = getHeight(x,z);

  scale = heightScale->pixels;

  if (heightScale->format->BitsPerPixel == 24) {
  color[0] = scale[3*(temp)];
  color[1] = scale[3*(temp)+1];
  color[2] = scale[3*(temp)+2];
  } else if (heightScale->format->BitsPerPixel == 32) {
  color[0] = scale[4*(temp)];
  color[1] = scale[4*(temp)+1];
  color[2] = scale[4*(temp)+2];
 }

}



/* sets height at a specific point */
int setHeight(unsigned short int x, unsigned short int z, unsigned char value)
{
 if( (x<tSIZE) && (z<tSIZE) ) 
  {
   heightMap[x+tSIZE*z] = value;
   return(0);
  }

#ifdef DEBUG
 else 
  {
/* this should not happen! */
   fprintf(debug,"debug: setHeight(%d,%d,%d) - x or y out of range!\n",x,z,value);
  }
#endif
   return(-1);

}
