/*
 genx realtime terrain visualization
 (c) 2004-2005 Witold Bolt <ja@hope.art.pl>

 file: main.c

 main file

*/

#include <stdio.h>
#include <stdlib.h>
#include <GL/gl.h>

#ifdef _WIN32
#  define WIN32_LEAN_AND_MEAN
#  include <windows.h>
#endif

#include "SDL.h"

#include "genx.h"
#include "normals.h"
#include "heightmap.h"
#include "initgl.h"
#include "error.h"
#include "csys.h"
#include "gsys.h"
#include "mesh.h"
#include "font.h"
#include "minimap.h"

FILE *debug;

SDL_bool fullscreen_mode = SDL_FALSE;

/* print up genx banner and opengl info */
static void show_banner()
{
 fprintf(debug,"genx version: %s by houp (ja@hope.art.pl)\n\n",GENX_VERSION);
}


/* main function :) */
#ifdef _WIN32
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	// Allows us to use WinMain() and bypass the stupid console window in Windows.
	int argc = __argc;
	char **argv = __argv;
#else
int main(int argc, char** argv)
{
#endif

 char title[40];          /* window title string */
 char *heightmap_filename=NULL;
 char *scale_filename=NULL;
 SDL_Event event;
 int loop=1;
 unsigned int i,width=0,height=0;

 debug=fopen("debug.txt","w");
 show_banner();

 if (argc<2)
  {
   fprintf(debug,"usage:\n\t%s --map [map_file] --width [window/screen width]\n",argv[0]);
   fprintf(debug,"example:\n\t %s --map mapa256.bmp --width 400\n",argv[0]);
   fprintf(debug,"note: maps must be in %s directory\n",MAPS_DIR);
   fprintf(debug,"note: height will be 3/4*width!\n");
  }

  for(i=1;i<argc;i++) {
  if(!strcmp(argv[i],"--width")) {
   if (argv[i+1]!=NULL) { width=strtol(argv[i+1],(char**)NULL,10);
			  height = 3 * width / 4;
			  if(height*width==0 || width+height<0)
			   { fprintf(debug,"ERROR: wrong width = %s\n",argv[i+1]);
			     exit(-1);
			   }
			}
    else { fprintf(debug,"ERROR: --width given with no value!\n");
           exit(-1);
         }
      i++;
  } 
  else if (!strcmp(argv[i],"--map")) {
   if (argv[i+1]!=NULL) { heightmap_filename=argv[i+1];
			}
    else { fprintf(debug,"ERROR: --map given with no value!\n");
           exit(-1);
         }
      i++;
  } 
 else if (!strcmp(argv[i],"--scale")) {
   if (argv[i+1]!=NULL) { scale_filename=argv[i+1];
			}
    else { fprintf(debug,"ERROR: --map given with no value!\n");
           exit(-1);
         }
      i++;
  } 
 else if (!strcmp(argv[i],"--fullscreen")) {
 fullscreen_mode=SDL_TRUE;
  } 
  
  
  else {

   fprintf(debug,"ERROR: unrecognized option: %s\n",argv[i]);
   }


   }

  if (heightmap_filename==NULL) {
  heightmap_filename = (char*)calloc((strlen(DEFAULT_MAP)+1),sizeof(char));
  strncpy(heightmap_filename,DEFAULT_MAP,strlen(DEFAULT_MAP));
  fprintf(debug," ---- using default map: %s ----\n",DEFAULT_MAP);
  }

  if (scale_filename==NULL) {
  scale_filename = (char*)calloc((strlen(DEFAULT_SCALE)+1),sizeof(char));
  strncpy(scale_filename,DEFAULT_SCALE,strlen(DEFAULT_SCALE));
  fprintf(debug," ---- using default scale: %s ----\n",DEFAULT_SCALE);
  }

  if(width==0) {
   fprintf(debug," ---- using default width: %d ----\n",DEFAULT_WIDTH);
   width=DEFAULT_WIDTH;
   height = 3 * width / 4;
   }

  if (getDataFromFile(heightmap_filename)!=0) exit(-1);

#ifdef DEBUG
 fprintf(debug,"debug: normals caltulation...\n");
#endif

  calculateNormals();

  if(initHeightScale(scale_filename)==-1) { SDL_Quit(); return(-1); }

//  initMiniMap(256);

  if(initGL(width,height)==-1) { fprintf(debug,"ERROR: InitGL() failed!\n"); 
  SDL_Quit(); return(-1); }

  sprintf(title,"terrain size: %d x %d",tSIZE,tSIZE);
  SDL_WM_SetCaption(title, "genx");

#ifdef DEBUG
 fprintf(debug,"debug: entering the main loop\n");
#endif

 SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,SDL_DEFAULT_REPEAT_INTERVAL);

 while(loop) {
  gSys_Draw();
     while ( SDL_PollEvent(&event) ) {	
          switch(event.type) {

        case SDL_MOUSEMOTION: 
        	cSys_MMotion();
        	break;

 	case SDL_QUIT: 
 		loop=0;
 	        break;

        case SDL_VIDEORESIZE: 
        	gSys_Reshape(event.resize.w,event.resize.h); 
                break;
        default:
        	cSys_Keyboard();
        	break;
        	
     }
    }

    }

 BMF_Free(genxFont);
 SDL_Quit();
 return(0);
}
