/*
 genx realtime terrain visualization
 (c) 2004-2005 Witold Bolt <ja@hope.art.pl>

 file: normals.c

 normal vectors calculations  - quite stupid code

*/


#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include <GL/gl.h>
#include "SDL.h"

#include "genx.h"
#include "normals.h"
#include "vector.h"
#include "heightmap.h"
#include "mesh.h"

/* tools function for setting normal vector values*/
static void setNormalVector(unsigned short int x, unsigned short int z, float vec[3])
{
 *(*(normalVectors[0]+z)+x) = vec[0];
 *(*(normalVectors[1]+z)+x) = vec[1];
 *(*(normalVectors[2]+z)+x) = vec[2];
}

/* puts specific normal vector in vec */
void getNormalVector(unsigned short int x, unsigned short int z, float vec[3])
{
 vec[0] = *(*(normalVectors[0]+z)+x);
 vec[1] = *(*(normalVectors[1]+z)+x);
 vec[2] = *(*(normalVectors[2]+z)+x);
}

/* allocte memory for normal vectors data */
void allocateNormals(unsigned short int size_x, unsigned short int size_z)
{
 unsigned int i,j;

 for (i = 0; i < 3; ++i) {
	normalVectors[i] = (float**)malloc(size_z*sizeof(float *));
	for (j = 0; j < size_z; ++j) {
		*(normalVectors[i] + j) = (float*)malloc(size_x*sizeof(float));
	}
 }
}


void freeNormals()
{
 int i;
 for (i = 0; i < 3; ++i) {
  free(normalVectors[i]); 
  }

 free(normalVectors);
}

/* calculates normal for height point x,z -> heightmap.c*/
void calculateNormal(unsigned short int x, unsigned short int z)
{
 float v1[3], v2[3], v3[3], v4[3], p1[3], p2[3],p3[3], p4[3];
 float result[3];

 p1[0] = x * SCALE_XZ;
 p1[1] = getHeight(x,z) * SCALE_Y;
 p1[2] = z * SCALE_XZ;

 p2[0] = (x)* SCALE_XZ;
 p2[1] = getHeight(x,z+1) * SCALE_Y;
 p2[2] = (z+1)* SCALE_XZ;

 makeVector(v3,p1,p2);

 p2[0] = (x)* SCALE_XZ;
 p2[1] = getHeight(x,z-1) * SCALE_Y;
 p2[2] = (z-1)* SCALE_XZ;

 makeVector(v1,p1,p2);

 p2[0] = (x+1)* SCALE_XZ;
 p2[1] = getHeight(x+1,z) * SCALE_Y;
 p2[2] = (z)* SCALE_XZ;

 makeVector(v2,p1,p2);

 p2[0] = (x-1)* SCALE_XZ;
 p2[1] = getHeight(x-1,z) * SCALE_Y;
 p2[2] = (z)* SCALE_XZ;

 makeVector(v4,p1,p2);

 crossProduct(p1, v1,v2);
 crossProduct(p2, v2,v3);
 crossProduct(p3, v4,v1);
 crossProduct(p4, v3,v4);


 reduceToUnit(p1);
 reduceToUnit(p2);
 reduceToUnit(p3);
 reduceToUnit(p4);

 result[0] = p1[0]+p2[0]+p3[0]+p4[0];
 result[1] = p1[1]+p2[1]+p3[1]+p4[1];
 result[2] = p1[2]+p2[2]+p3[2]+p4[2];

 if(result[1]<0.0f) {
  
  result[0]*=(-1.0f);
  result[1]*=(-1.0f);
  result[2]*=(-1.0f);
 }
 reduceToUnit(result);

 setNormalVector(x,z,result);
}

/* calculate all of the normal vectors */
void calculateNormals()
{
 int x,z;
 float temp[3];

 allocateNormals(tSIZE,tSIZE);

 for(x=1;x<tSIZE-1; x++)
  for(z=1;z<tSIZE-1; z++)
    calculateNormal(x,z);

/* we calculate borders separately */

  getNormalVector(tSIZE-2,tSIZE-2,temp);
  setNormalVector(tSIZE-1,tSIZE-1,temp);

  getNormalVector(1,1,temp);
  setNormalVector(0,0,temp);

 for(x=1;x<tSIZE-1; x++)
 {
  getNormalVector(x,tSIZE-2,temp);
  setNormalVector(x,tSIZE-1,temp);

  getNormalVector(x,1,temp);
  setNormalVector(x,0,temp);
 }

 for(z=1;z<tSIZE-1;z++)
 {
  getNormalVector(tSIZE-2,z,temp);
  setNormalVector(tSIZE-1,z,temp);

  getNormalVector(1,z,temp);
  setNormalVector(0,z,temp);

 }



}

#ifdef DEBUG
/* for testing only */
void printNormal(unsigned short int x, unsigned short int z)
{
  float temp[3];
  getNormalVector(x,z,temp);
  fprintf(debug,"normal(%d,%d) = [ %f, %f, %f ]\n",x,z,temp[0],temp[1],temp[2]);
}
#endif
