/*
 genx realtime terrain visualization
 (c) 2004-2005 Witold Bolt <ja@hope.art.pl>

 file: sky.c

 sky box rendeing...

 some code for sky box rendering came from: 
 height map 2 program by DigiBen (or rather from the SDL port of his program)
 found at: http://www.gametutorials.com/Tutorials/opengl/OpenGL_Pg4.htm
*/

#include <GL/gl.h>
#include <GL/glext.h>
#include <GL/glu.h>

#include "SDL.h"
#include "SDL_image.h"

#include "genx.h"
#include "sky.h"
#include "error.h"
#include "gsys.h"

#define SIDE 0
#define TOP 1

GLuint skyTexture[2];

SDL_bool sky_mode=SDL_TRUE;

static GLint PrepareTexture(char *filename)
{
  SDL_Surface *ImageName;
  GLint texID;
  GLenum format=GL_RGB;

  ImageName = IMG_Load(filename);
  if (!ImageName) {
  fprintf(debug,"ERROR: loading texture file %s failed with error: %s\n",filename,IMG_GetError());
  return(-1);
  }
#ifdef DEBUG
  else {
  fprintf(debug,"debug: file %s info:\n",filename);
  fprintf(debug,"debug: width: %d, height: %d, bpp: %d\n",ImageName->w,ImageName->h,ImageName->format->BitsPerPixel);
  }
#endif

  if(ImageName->format->BitsPerPixel == 24) format=GL_RGB;
  else if(ImageName->format->BitsPerPixel == 32) format=GL_RGBA;
  else { fprintf(debug,"ERROR: file: %s is in unsupported pixel format!\n",filename);
         return(-1);
       }


  glGenTextures(1, &texID);
  glBindTexture(GL_TEXTURE_2D, texID);
  glTexImage2D(GL_TEXTURE_2D, 0,ImageName->format->BytesPerPixel, ImageName->w, ImageName->h,0, format, GL_UNSIGNED_BYTE, ImageName->pixels);

  SDL_FreeSurface(ImageName);
  process_errors();

  return(texID);
}

GLint prepareSkyTextures()
{
 GLuint texture;
 char filename[255];

 sprintf(filename,"%s/%s",TEXTURES_DIR,SKY_SIDE_TEXTURE);
 
 texture = PrepareTexture(filename);
 if (texture!=-1)  skyTexture[SIDE] = texture;
 else return(texture);

 sprintf(filename,"%s/%s",TEXTURES_DIR,SKY_TOP_TEXTURE);

 texture = PrepareTexture(filename);
 if (texture!=-1)  skyTexture[TOP] = texture;
 else return(texture);

 return(0);
}


void putSkyBox(float x, float y, float z, float width, float height, float length)
{

    glNewList(SKY_DISPLAY_LIST,GL_COMPILE);

    // This centers the sky box around (x, y, z)
    x = x - width  / 2;
    y = y - height / 2;
    z = z - length / 2;

    // Start drawing the side as a QUAD
    glBindTexture(GL_TEXTURE_2D, skyTexture[SIDE]);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    glBegin(GL_QUADS);      
    glColor4f(1.0f,1.0f,1.0f,1.0f);
        // Assign the texture coordinates and vertices for the BACK Side
        glTexCoord2f(1.0f, 0.0f); glVertex3f(x + width, y,          z);
        glTexCoord2f(1.0f, 1.0f); glVertex3f(x + width, y + height, z); 
        glTexCoord2f(0.0f, 1.0f); glVertex3f(x,         y + height, z);
        glTexCoord2f(0.0f, 0.0f); glVertex3f(x,         y,          z);
    glEnd();

    // Start drawing the side as a QUAD
    glBegin(GL_QUADS);  
    glColor4f(1.0f,1.0f,1.0f,1.0f);
        // Assign the texture coordinates and vertices for the FRONT Side
        glTexCoord2f(1.0f, 0.0f); glVertex3f(x,         y,          z + length);
        glTexCoord2f(1.0f, 1.0f); glVertex3f(x,         y + height, z + length);
        glTexCoord2f(0.0f, 1.0f); glVertex3f(x + width, y + height, z + length); 
        glTexCoord2f(0.0f, 0.0f); glVertex3f(x + width, y,          z + length);
    glEnd();

    // Start drawing the side as a QUAD
    glBegin(GL_QUADS);      
    glColor4f(1.0f,1.0f,1.0f,1.0f);
        // Assign the texture coordinates and vertices for the LEFT Side
        glTexCoord2f(1.0f, 1.0f); glVertex3f(x,         y + height, z); 
        glTexCoord2f(0.0f, 1.0f); glVertex3f(x,         y + height, z + length); 
        glTexCoord2f(0.0f, 0.0f); glVertex3f(x,         y,          z + length);
        glTexCoord2f(1.0f, 0.0f); glVertex3f(x,         y,          z);     
    glEnd();

    // Start drawing the side as a QUAD
    glBegin(GL_QUADS);      
    glColor4f(1.0f,1.0f,1.0f,1.0f);
        // Assign the texture coordinates and vertices for the RIGHT Side
        glTexCoord2f(0.0f, 0.0f); glVertex3f(x + width, y,          z);
        glTexCoord2f(1.0f, 0.0f); glVertex3f(x + width, y,          z + length);
        glTexCoord2f(1.0f, 1.0f); glVertex3f(x + width, y + height, z + length); 
        glTexCoord2f(0.0f, 1.0f); glVertex3f(x + width, y + height, z);
    glEnd();

    // Start drawing the side as a QUAD
    glBindTexture(GL_TEXTURE_2D, skyTexture[TOP]);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    glBegin(GL_QUADS);      
    glColor4f(1.0f,1.0f,1.0f,1.0f);
        // Assign the texture coordinates and vertices for the TOP Side
        glTexCoord2f(1.0f, 0.0f); glVertex3f(x + width, y + height, z);
        glTexCoord2f(1.0f, 1.0f); glVertex3f(x + width, y + height, z + length); 
        glTexCoord2f(0.0f, 1.0f); glVertex3f(x,         y + height, z + length);
        glTexCoord2f(0.0f, 0.0f); glVertex3f(x,         y + height, z);
    glEnd();


   glEndList();
}
